//Krb4Test.java


import java.io.*;
import java.util.*;
import java.net.*;
import krb4.lib.*;
import krb4.lib.crypto.*;

public class Krb4Test {

	public static void usage() {
		System.out.println("java Krb4Test <ap_req binary file>");
		System.out.println("e.g.: java Krb4Test ap_req.bin");
	}

	public static void main(String[] args) throws IOException {
		Krb4APReq ap_req = null;
		try {
			//ap_req binary file specified on command line

			if (args.length != 1 || args[0] == null) {
				usage();
				return;
			}

			File f = new File(args[0]);
			if (!f.exists()) {
				usage();
				return;
			}

			FileInputStream i = new FileInputStream(args[0]);
			byte[] apReqBytes = new byte[i.available()];
			i.read(apReqBytes);
			i.close();

			ap_req = new Krb4APReq(apReqBytes);

			//lets you know if you have to generate a response
			System.out.println("mutual auth required = " +
				ap_req.mutualAuthRequired());

			//these methods help you select a key
			String srealm = ap_req.getServerRealm();
			System.out.println("requested server realm = " +
				srealm); 
			System.out.println("requested key version = " +
				ap_req.getServerKeyVersion());

			byte[] key = des.string_to_key_bytes("$tester");
			ap_req.authenticate(
				key, //byte[8]
				"tt10-demo", //server name
				"", //server instance
				"CIT.CORNELL.EDU", //server realm
				null //InetAddress.getByName("bluesky.osf.org").getAddress() //byte[4] caddr
				);
			//or just...
			//ap_req.authenticate(key);

			System.out.print("client = " + ap_req.getClientName());
			String cinst = ap_req.getClientInstance();
			if (cinst.length() > 0)
				System.out.print("." + cinst);
			System.out.println("@" + ap_req.getClientRealm());

			System.out.print("server = " + ap_req.getServerName());
			String sinst = ap_req.getServerInstance();
			if (sinst.length() > 0)
				System.out.print("." + sinst);
			System.out.println("@" + srealm);

		} catch(Krb4Exception e) {
			try {
				System.out.println(new Date(ap_req.authenticator.timestamp));
			} catch (Exception e2) {}
			System.out.println(e.krbErrorMessage());
		} catch(Exception e) {
			System.out.println(e);
		}
	}

}