// Copyright 1997 The Open Group Research Institute.  All rights reserved.

package krb4.lib;

import java.io.*;
import java.net.*;
import krb4.lib.crypto.*;

public class Krb4ADMReq {
	public byte pvno;
	public byte msgType;
	boolean littleEndian;
	public byte keyVersion;
	public String srealm;
	public byte[] encTicket;
	public byte[] encAuthenticator;
	public long timestamp;
	public long lifetime;
	public String sname;
	public String sinst;
	public byte[] obuf;
	public byte[] ibuf;
	Krb4Creds creds;
	public Krb4KRBPriv privData;
	public byte[] privBytes;
	public TCPClient client;
	public static String version = "KADM0.0A";

	public Krb4ADMReq(
		Krb4Creds new_creds,
		String cname,
		String cinst,
		String crealm,
		Krb4KRBPriv new_privData,
		int new_keyVersion,
		TCPClient new_client,
		String new_srealm,
		long new_timestamp,
		long new_lifetime,
		String new_sname,
		String new_sinst
	) throws Krb4Exception, IOException {

	    creds = new_creds;
	    pvno = (byte)Krb4.KRB_PROT_VERSION;
	    msgType = (byte)Krb4.AUTH_MSG_APPL_REQUEST;
	    littleEndian = false;
	    keyVersion = (byte)new_keyVersion;
	    srealm = new_srealm;
	    privData = new_privData;
	    encTicket = creds.encTicket;
	    timestamp = new_timestamp;
	    lifetime = new_lifetime;
	    client = new_client;
	    sname = new_sname;
	    sinst = new_sinst;
	    byte[] temp;
	    privBytes = privData.encode();
	    long chksum = des.des_quad_cksum(privBytes, (byte[])null, (long)privBytes.length, (int)0, creds.sessionKey);

	    Krb4APReq ap_req = new Krb4APReq(
					     creds,
					     cname,
					     cinst,
					     crealm,
					     (int)chksum,
					     false //No mutual auth;
					     );

	    ap_req.littleEndian = false;
	    encAuthenticator = ap_req.encode();
	    obuf = encode();
	}

	public byte[] encode() throws Krb4Exception, IOException {
		ByteArrayOutputStream o = new ByteArrayOutputStream();
		o.write(version.getBytes()); //new;
		o.write(Krb4Encode.toBytes((int)privBytes.length,false));
		o.write(encAuthenticator);
		o.write(privBytes);

		return o.toByteArray();
	}


	public void send() throws IOException {
	        //we could use a Krb4Encode.toBytes(short) here;
		short size = (short)obuf.length;
		byte[] temp = new byte[2];
		temp[1] = (byte)(size & 0xff);
		temp[0] = (byte)((size >>> 8) & 0xff);
	        client.send(temp); // send the size first
		  
		client.send(obuf);

		temp = client.receive(2); // read the size first;
		int i = (int)temp[0]*256 + (int)temp[1];
		ibuf = client.receive(i); // read the data;
		return;
	}

	public Krb4ADMRep getKrb4ADMRep() throws Krb4Exception, IOException {
		Krb4ADMRep rep = null;
		try {
			rep = new Krb4ADMRep(ibuf, creds, client);
		} catch (Krb4Exception e) {
			rep = null;
			if (e.returnCode() == Krb4.RD_AP_MSG_TYPE) {
				Krb4KDCErr err = new Krb4KDCErr(ibuf);
				throw new Krb4Exception(err.error_code);
			}
			else
				throw e;
		}
		return rep;
	}
}
