//SecureTCPClient.java

import java.io.*;
import java.net.*;
import krb5.lib.*;
import krb5.lib.ccache.*;

class SecureTCPClient {

	public static void main(String args[]) throws UnknownHostException {

		String principal = "me@MEMESOFT.COM";
		StringBuffer password = new StringBuffer("me");
		String kdc = "phage.this.net";
		String service = "myserver@MEMESOFT.COM";
		InetAddress address = InetAddress.getByName("phage.this.net");
		int port = 8891;
		TCPClient client = null;

		//generate a specific subSessionKey from a character string
		//String subKeyString = "sdfsdffg";
		//byte[] subKeyData = new byte[subKeyString.length()];
		//subKeyString.getBytes(0, subKeyString.length(), subKeyData, 0);
		//EncryptionKey subSessionKey = new EncryptionKey(subKeyData);

		//or...

		//specify no subSessionKey
		EncryptionKey subSessionKey = null;

		try {

            Config.setKDCHostName(kdc);
            CCacheFactory ccacheFactory = CCacheFactory.getFactory();
            CredentialsCache ccache = ccacheFactory.newCCacheInstance("VmCCacheImpl");
            ccache.getInitialCredentials(principal, password);
            Credentials serviceCreds = ccache.getServiceCredentials(service);
			
			client = new TCPClient(address, port);

			String str = " 01234567ABCDEFGHello; this is a secure message!";
			byte[] userData = new byte[str.length()];
			str.getBytes(0, str.length(), userData, 0);

			LocalSeqNumber seqNumber = new LocalSeqNumber();

			KrbApReq ap_req = new KrbApReq(
				new APOptions(Krb5.AP_OPTS_MUTUAL_REQUIRED),
				serviceCreds,
				null, //Checksum cksum
				subSessionKey,
				seqNumber,
				null  //AuthorizationData authzData
			);

			System.out.println("send ap_req");

			client.send(ap_req.msg());
			byte[] ibuf = client.receive();

			System.out.println("received ap_rep");

			KrbApRep ap_rep = new KrbApRep(
				ibuf,
				serviceCreds
			);
			ap_rep.authenticate(ap_req);
			if (ap_rep.getSubKey() != null)
				subSessionKey = ap_rep.getSubKey();

			System.out.println("authenticated");
			System.out.println("server=\"" + serviceCreds.server() + "\"");
			System.out.println();

			do {

				System.out.println("userData=\"" + new String(userData, 0) + "\"");

				KrbMkPriv krb_priv = new KrbMkPriv(
					userData,
					serviceCreds,
					subSessionKey,
					new KerberosTime(KerberosTime.NOW),
					seqNumber,
					new HostAddress(),
					new HostAddress(InetAddress.getLocalHost())
				);

				System.out.println("send");

				client.send(krb_priv.msg());
				ibuf = client.receive();

				System.out.println("received");

				KrbRdPriv krb_priv_reply = new KrbRdPriv(
					ibuf,
					serviceCreds,
					subSessionKey,
					seqNumber,
					new HostAddress(client.getInetAddress()),
					new HostAddress(),
					true, //boolean timestampRequired
					true //boolean seqNumberRequired
				);

				System.out.println("decrypted");

				byte[] replyData = krb_priv_reply.data();

				System.out.println("replyData=\"" + new String(replyData, 0) + "\"");
				System.out.println();

				//why wait..
				//try {
				//	Thread.currentThread().sleep(1000);
				//} catch (InterruptedException e) { }

			}
			while (true);

		}
		catch (KrbException e) {
			e.printStackTrace();
			System.out.println(e.krbErrorMessage());
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		finally {
			try {
				if (client != null)
					client.close();
			}
			catch (IOException e) {
			}
		}

		System.out.println("ok");
	}
}
