// Copyright 1997 The Open Group Research Institute.  All rights reserved.

package krb5.lib;

import krb5.lib.asn1.*;

/**
 * Encryption type entry
 */
public class ETypeInfoEntry {

	/**
	 * Encryption type
	 */
	public int eType;

	/**
	 * Salt
	 */
	public byte[] salt; //optional

	/**
	 * Class constructor
	 *
	 * @param ref is of type EncodeRef
	 * @exception Asn1Exception an exception
	 * @see krb5.lib.asn1.EncodeRef
	 * @see krb5.lib.Asn1Exception
	 */
	public ETypeInfoEntry(EncodeRef ref) throws Asn1Exception {
		EncodeRef subRef = decode.Sequence(ref);
		if (decode.Tag(subRef) == 0) {
			eType = decode.Integer(subRef.startOfData());
			//XXX need to validate that the type is recognized
			subRef.next();
		}
		else
			throw new Asn1Exception(Krb5.ASN1_BAD_ID);
		if (decode.Tag(subRef) == 1) {
			salt = decode.OctetString(subRef.startOfData());
			//XXX need to validate salt is right for eType
			subRef.next();
		}
		if (subRef.isMoreInSequence())
			throw new Asn1Exception(Krb5.ASN1_BAD_ID);
	}

	/**
	 * Encodes the object in asn1
	 *
	 * @return byte[] is a result
	 * @exception Asn1Exception an exception
	 * @see krb5.lib.Asn1Exception
	 */
	public byte[] asn1Encode() throws Asn1Exception {
		if (salt != null) {
			byte[][] etype_info_entry = {
				encode.prependExplicitTag(0, encode.Integer(eType)),
				encode.prependExplicitTag(1, encode.OctetString(salt))
			};
			return encode.Sequence(etype_info_entry);
		}
		else {
			byte[][] etype_info_entry = {
				encode.prependExplicitTag(0, encode.Integer(eType)),
			};
			return encode.Sequence(etype_info_entry);
		}
	}

}