// Copyright 1997 The Open Group Research Institute.  All rights reserved.

package krb5.lib.crypto;

import krb5.lib.*;

public abstract class CksumType {

	public static CksumType getInstance(int cksumTypeConst)
		throws KdcErrException {
		CksumType cksumType = null;
		switch (cksumTypeConst) {
			case Krb5.CKSUMTYPE_CRC32:
				cksumType = new Crc32CksumType();
				break;
			case Krb5.CKSUMTYPE_RSA_MD4:
				cksumType = new RsaMd4CksumType();
				break;
			case Krb5.CKSUMTYPE_RSA_MD4_DES:
				cksumType = new RsaMd4DesCksumType();
				break;
			case Krb5.CKSUMTYPE_DES_MAC:
				cksumType = new DesMacCksumType();
				break;
			case Krb5.CKSUMTYPE_DES_MAC_K:
				cksumType = new DesMacKCksumType();
				break;
			case Krb5.CKSUMTYPE_RSA_MD4_DES_K:
				cksumType = new RsaMd4DesKCksumType();
				break;
			case Krb5.CKSUMTYPE_RSA_MD5:
				cksumType = new RsaMd5CksumType();
				break;
			case Krb5.CKSUMTYPE_RSA_MD5_DES:
				cksumType = new RsaMd5DesCksumType();
				break;
			default:
				throw new KdcErrException(Krb5.KDC_ERR_SUMTYPE_NOSUPP);
		}
		return cksumType;
	}

	public static CksumType getInstance() throws KdcErrException {
		return getInstance(Config.CKSUMTYPE_DEFAULT);
	}

	public abstract int confounderSize();

	public abstract int cksumType();

	public abstract boolean isSafe();

	public abstract int cksumSize();

	public abstract int keyType();

	public abstract int keySize();

	public abstract byte[] calculateChecksum(byte[] data, int size);

	public abstract byte[] calculateKeyedChecksum(byte[] data, int size,
		byte[] key);

	public abstract boolean verifyKeyedChecksum(byte[] data, int size,
		byte[] key, byte[] checksum);

	public boolean isChecksumEqual(byte[] cksum1, byte[] cksum2) {
		if (cksum1 == cksum2)
			return true;
		if ((cksum1 == null && cksum2 != null) ||
			(cksum1 != null && cksum2 == null))
			return false;
		if (cksum1.length != cksum2.length)
			return false;
		for (int i = 0; i < cksum1.length; i++)
			if (cksum1[i] != cksum2[i])
				return false;
		return true;
	}

}
