// Copyright 1997 The Open Group Research Institute.  All rights reserved.

package krb5.lib.crypto;

import krb5.lib.*;

public class RsaMd4DesKCksumType extends CksumType {

	public RsaMd4DesKCksumType() {
	}

	public int confounderSize() {
		return 0;
	}

	public int cksumType() {
		return Krb5.CKSUMTYPE_RSA_MD4_DES_K;
	}

	public boolean isSafe() {
		return true;
	}

	public int cksumSize() {
		return 16;
	}

	public int keyType() {
		return Krb5.KEYTYPE_DES;
	}

	public int keySize() {
		return 8;
	}

	public byte[] calculateChecksum(byte[] data, int size) {
		return null;
	}

	public byte[] calculateKeyedChecksum(byte[] data, int size, byte[] key) {
		//calculate md4 cksum
		md4 mdc = new md4(data, size);
		mdc.calc();
		byte[] cksum = mdc.toBytes();

		//XXX need to check for weak keys
		long[] key_sched = new long[16];
		des.des_set_key(key, key_sched);
		//set ivec to key
		byte[] ivec = new byte[key.length];
		System.arraycopy(key, 0, ivec, 0, key.length);

		//des-cbc encrypt
		byte[] enc_cksum = new byte[cksum.length];
		//XXX would like it to work in place
		des.cbc_encrypt(cksum, enc_cksum, key_sched, ivec, true);

		//return value
		return enc_cksum;
	}

	public boolean verifyKeyedChecksum(byte[] data, int size,
		byte[] key, byte[] checksum) {
		//decrypt checksum
		byte[] cksum = decryptKeyedChecksum(checksum, key);

		//calculate md4 cksum
		md4 mdc = new md4(data, size);
		mdc.calc();
		byte[] new_cksum = mdc.toBytes();

		return isChecksumEqual(cksum, new_cksum);
	}

	private byte[] decryptKeyedChecksum(byte[] enc_cksum, byte[] key) {
		//XXX need to check for weak keys
		long[] key_sched = new long[16];
		des.des_set_key(key, key_sched);
		//set ivec to key
		byte[] ivec = new byte[key.length];
		System.arraycopy(key, 0, ivec, 0, key.length);

		//des-cbc encrypt
		byte[] cksum = new byte[enc_cksum.length];
		//XXX would like it to work in place
		des.cbc_encrypt(enc_cksum, cksum, key_sched, ivec, false);

		//return value
		return cksum;
	}

}
