// Copyright 1997 The Open Group Research Institute.  All rights reserved.

package krb5.lib.crypto;

import krb5.lib.*;

public class RsaMd5DesCksumType extends CksumType {

	public RsaMd5DesCksumType() {
	}

	public int confounderSize() {
		return 8;
	}

	public int cksumType() {
		return Krb5.CKSUMTYPE_RSA_MD5_DES;
	}

	public boolean isSafe() {
		return true;
	}

	public int cksumSize() {
		return 24;
	}

	public int keyType() {
		return Krb5.KEYTYPE_DES;
	}

	public int keySize() {
		return 8;
	}

	public byte[] calculateChecksum(byte[] data, int size) {
		return null;
	}

	public byte[] calculateKeyedChecksum(byte[] data, int size, byte[] key) {
		//prepend confounder
		byte[] new_data = new byte[size + confounderSize()];
		byte[] conf = Confounder.bytes(confounderSize());
		System.arraycopy(conf, 0, new_data, 0, confounderSize());
		System.arraycopy(data, 0, new_data, confounderSize(), size);

		//calculate md5 cksum
		MD5 mdc = new MD5();
		mdc.Update(new_data, new_data.length);
		byte[] mdc_cksum = mdc.Final();
		byte[] cksum = new byte[cksumSize()];
		System.arraycopy(conf, 0, cksum, 0, confounderSize());
		System.arraycopy(mdc_cksum, 0, cksum, confounderSize(),
			cksumSize() - confounderSize());

		//compute modified key
		byte[] new_key = new byte[keySize()];
		System.arraycopy(key, 0, new_key, 0, key.length);
		for (int i = 0; i < new_key.length; i++)
			new_key[i] = (byte)(new_key[i] ^ 0xf0); 
		//XXX need to check for weak keys
		long[] key_sched = new long[16];
		des.des_set_key(new_key, key_sched);
		byte[] ivec = new byte[new_key.length];

		//des-cbc encrypt
		byte[] enc_cksum = new byte[cksum.length];
		//XXX would like it to work in place
		des.cbc_encrypt(cksum, enc_cksum, key_sched, ivec, true);

		//return value
		return enc_cksum;
	}

	public boolean verifyKeyedChecksum(byte[] data, int size,
		byte[] key, byte[] checksum) {
		//decrypt checksum
		byte[] cksum = decryptKeyedChecksum(checksum, key);

		//prepend confounder
		byte[] new_data = new byte[size + confounderSize()];
		System.arraycopy(cksum, 0, new_data, 0, confounderSize());
		System.arraycopy(data, 0, new_data, confounderSize(), size);

		//calculate md5 cksum
		MD5 mdc = new MD5();
		mdc.Update(new_data, new_data.length);
		byte[] new_cksum = mdc.Final();

		//extract original cksum value
		byte[] orig_cksum = new byte[cksumSize() - confounderSize()];
		System.arraycopy(cksum,  confounderSize(), orig_cksum, 0,
			cksumSize() - confounderSize());

		return isChecksumEqual(orig_cksum, new_cksum);
	}

	private byte[] decryptKeyedChecksum(byte[] enc_cksum, byte[] key) {
		//compute modified key
		byte[] new_key = new byte[keySize()];
		System.arraycopy(key, 0, new_key, 0, key.length);
		for (int i = 0; i < new_key.length; i++)
			new_key[i] = (byte)(new_key[i] ^ 0xf0); 
		//XXX need to check for weak keys
		long[] key_sched = new long[16];
		des.des_set_key(new_key, key_sched);
		byte[] ivec = new byte[new_key.length];

		//des-cbc encrypt
		byte[] cksum = new byte[enc_cksum.length];
		//XXX would like it to work in place
		des.cbc_encrypt(enc_cksum, cksum, key_sched, ivec, false);

		//return value
		return cksum;
	}

}
